/*
  Reversing loop
  Reversing loop for model railroads
  Uses 2 IR detectors, 3 LEDs and 1 (or more) relay
  
  Pin 8: IR detector left (0 = free, 1 = occupied)
  Pin 9: IR detector right (0 = free, 1 = occupied)
  Pin 11: yellow LED left (standard polarity - unwatched gap is safe to pass)
  Pin 13: green LED status (flash during timedelay)
  Pin 12: yellow LED right (reversed polarity - gap with detectors is safe to pass)
  Pin 10: Relay(s)

  Weerstand voor vertraging?
  Jumper voor relais omdraaien?
 
   Huib Maaskant - www.floodland.nl/aim
 */

// Name pins
int detectl = 8;
int detectr = 9;
int ledyl = 11;
int ledg = 13;
int ledyr = 12;
int relay = 10;
int potpin = 0;            // analog pin used to connect the potentiometer

// Variables
int detectstatus;          // detection status: 1 = one or more detectors occupied
unsigned long waituntil;   // time out counter
unsigned long ledflash;    // led flash counter
int timedelay;             // delay in millisecs

// the setup routine runs once when you press reset:
void setup() {                
  // initialize pins
  pinMode(detectl, INPUT);
  pinMode(detectr, INPUT);
  pinMode(ledyl, OUTPUT);
  pinMode(ledg, OUTPUT);
  pinMode(ledyr, OUTPUT);
  pinMode(relay, OUTPUT);
  
  detectstatus = 0;
  ledflash = 0;
  Serial.begin(9600);
}

// the loop routine runs over and over again forever:
void loop() {

  // idle state - standard polarity
  digitalWrite(ledg, HIGH);
  digitalWrite(ledyl, HIGH);
  digitalWrite(ledyr, LOW);
  digitalWrite(relay, LOW);

  // wait for detection
  while (detectstatus == 0) {
    detectstatus = digitalRead (detectl) + digitalRead (detectr);
  }
  
  // train approaching the gap with detection
  
  // change polarity
  digitalWrite(ledyl, LOW);
  digitalWrite(ledyr, HIGH);
  digitalWrite(relay, HIGH);
  
   // wait for detection to clear and timedelay to run out
  timedelay = map(analogRead(potpin), 0, 1023, 10, 5000);     // delay before polarity is changed to normal after clearing detection (millisecs)
  waituntil = millis() + timedelay; 
  
  while (waituntil > millis()) {
    // read status of detectors
    detectstatus = digitalRead (detectl) + digitalRead (detectr);

    if (detectstatus !=0) {
        // reset time counter if detector still occupied
        waituntil = millis() + timedelay;
        // make green LED solid
        digitalWrite(ledg, HIGH);
    }
    else { // we're in the time delay: flash that green LED
        if (ledflash < millis()) {
          ledflash = millis() + 200; // set flash interval
          digitalWrite(ledg, !digitalRead(ledg));
        }  
    } // else
  }
  
  // train has cleared the gap with detection

}
